//+------------------------------------------------------------------+
//|                                                      ALMA_v1.mq5 |
//| ALMA by Arnaud Legoux / Dimitris Kouzis-Loukas / Anthony Cascino |
//|                                             www.arnaudlegoux.com |                     
//|                         Written by IgorAD,igorad2003@yahoo.co.uk |   
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |                                      
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"
#property description "Arnaud Legoux Moving Average"

#property indicator_chart_window

#property indicator_buffers 3
#property indicator_plots   1
#property indicator_type1   DRAW_COLOR_LINE
#property indicator_color1  Turquoise,OrangeRed,Linen
#property indicator_width1  2

#property indicator_label1  "ALMA"
#property indicator_applied_price PRICE_CLOSE

input int     WindowSize        =   9;    //Window Size  
input double  Sigma             = 6.0;    //Sigma parameter 
input double  Offset            =0.85;    //Offset of Gaussian distribution (0...1)
input int     Shift             =   0;    //Shift in Bars
input int     ColorMode         =   0;    //Color Mode(0-off,1-on) 

double  ALMA[];
double  ColorBuf[];

double  W[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,ALMA,INDICATOR_DATA);
   PlotIndexSetInteger(0,PLOT_COLOR_INDEXES,3);
   SetIndexBuffer(1,ColorBuf,INDICATOR_COLOR_INDEX);
//--- sets first bar from what index will be drawn
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,WindowSize);
//--- sets indicator shift
   PlotIndexSetInteger(0,PLOT_SHIFT,Shift);
//--- name for indicator label
   IndicatorSetString(INDICATOR_SHORTNAME,"ALMA("+string(WindowSize)+","+DoubleToString(Sigma,2)+","+DoubleToString(Offset,2)+")");
//--- name for index label
   PlotIndexSetString(0,PLOT_LABEL,"ALMA("+string(WindowSize)+","+DoubleToString(Sigma,2)+","+DoubleToString(Offset,2)+")");
   
   ArrayResize(W,WindowSize);
  
   double m = MathFloor(Offset * (WindowSize - 1));
	double s = WindowSize/Sigma;
	
	ArrayResize(W,WindowSize);
			
	double wSum = 0;
	for (int i=0;i < WindowSize;i++) 
	{
	W[i] = MathExp(-((i-m)*(i-m))/(2*s*s));
   wSum += W[i];
   } 
   
   for (int i=0;i < WindowSize;i++) W[i] = W[i] / wSum; 
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const int begin,
                const double &price[])
{
   if(rates_total < WindowSize) return(0);
   
   int limit=prev_calculated-1;
   
   if(limit < WindowSize) limit = WindowSize;
   
   int shift = limit;
   
   for(shift=limit;shift < rates_total;shift++)
   {
   double sum = 0.0;
   for(int i = 0; i < WindowSize; i++) sum += price[shift-(WindowSize - 1 - i)]*W[i];
   
   ALMA[shift] = sum;
   
      if (ColorMode > 0)
      {
         if(shift > 0)
         {
         //ColorBuf[shift] = 2;
         if (ALMA[shift] > ALMA[shift-1]) ColorBuf[shift] = 0;
         if (ALMA[shift] < ALMA[shift-1]) ColorBuf[shift] = 1;
         }
      }
      else
      ColorBuf[shift] = 2;
      
   }
   
   return(rates_total);
}
//+------------------------------------------------------------------+
